#!/bin/bash

base_path=$(cd $(dirname $0i); pwd)
cd $base_path

source ./config

# 信任FounderCA证书
if [ ! -e "/etc/pki/ca-trust/source/anchors/FounderCA.cer" ]; then
    cp FounderCA.cer /etc/pki/ca-trust/source/anchors/
    update-ca-trust
fi

# 判断证书和密钥是否存在
if [ ! -e $cert_path ]; then
    echo "文件不存在，请检查证书路径是否正确, $cert_path"
    exit 1
fi
if [ ! -e $key_path ]; then
    echo "文件不存在，请检查密钥路径是否正确, $cert_path"
    exit 1
fi

# 判断证书过期
current_date=$(date +%s)
cert_expiry_date=$(date +%s -f <(openssl x509 -enddate -noout -in /usr/local/nginx/conf/server.cer | awk -F = '{print $2}'))
left_days=$(( ($cert_expiry_date - $current_date) / 86400 ))

if [ $left_days -gt $before_days ]; then
    echo "证书剩余天数为 $left_days 天，无需重新签署证书"
    exit 0
fi

echo "证书剩余天数为 $left_days 天，需要重新签署证书"

# 处理SAN DNS
if [ -z $san_dnss ]; then
    san_dnss=("localhost" $(hostname))
else
    IFS=',' read -r -a san_dnss <<< "$san_dnss"
fi

# 处理SAN IP
if [ -z $san_ips ]; then
    san_ips=$(hostname -I)
    san_ips=("127.0.0.1" "${san_ips[@]}")
else
    IFS=',' read -r -a san_ips <<< "$san_ips"
fi

# 重新生成证书
rm -f server.*
openssl genrsa -out server.key 2048
openssl req -new -sha256 -out server.csr -key server.key -subj "/C=CN/ST=BeiJing/L=HaiDian/O=Founder/OU=TSD/CN=cnis.tsd.pku-hit.com"
openssl x509 -req -in server.csr -out server.cer -CAkey FounderIntermediateCA.key -CA FounderIntermediateCA.cer -days 90 -CAcreateserial -sha256 -extensions x509v3_server -passin pass:Founder123 -extfile <(cat <<EOF
[x509v3_server]
basicConstraints = CA:FALSE
extendedKeyUsage = serverAuth,clientAuth
subjectAltName = @alt_names

[alt_names]
$(for i in ${!san_dnss[@]}; do echo "DNS.$((i+1))=${san_dnss[i]}"; done;)
$(for i in ${!san_ips[@]}; do echo "IP.$((i+1))=${san_ips[i]}"; done;)
EOF
)

# 覆盖现有的证书
cp -f server.cer $cert_path
cat FounderIntermediateCA.cer >> $cert_path
cp -f server.key $key_path


# 让nginx重新加载配置
$nginx_path -s reload

echo '证书已经重新签署并Nginx已重新加载'
